@@ -389,35 +389,154 @@
 #' @return nothing
 #' @noRd
 
-.ddg.create.data.use.edges <- function (cmd, for.caller) {
+.ddg.create.data.use.edges <- function (cmd, for.caller, env=NULL) {
   # Find all the variables used in this command.
   #print (paste(".ddg.create.data.use.edges: cmd = ", cmd@text))
   vars.used <- cmd@vars.used
 
   for (var in vars.used) {
+    var <- .ddg.extract.var (var, env)
+    
     #print(paste(".ddg.create.data.use.edges: var =", var))
     # Make sure there is a node we could connect to.
-    scope <- .ddg.get.scope(var, for.caller)
+    scope <- .ddg.get.scope(var, for.caller, env=env)
 
     #print(paste(".ddg.create.data.use.edges: scope =", scope))
 
     if (.ddg.data.node.exists(var, scope)) {
-      # print(".ddg.create.data.use.edges found data node")
+      #print(".ddg.create.data.use.edges found data node")
       .ddg.data2proc(var, scope, cmd@abbrev)
     }
     else {
-      # Note: This generates lots of error nodes of limited value.
-      # Turn off for now.
-
-      # error.msg <- paste("Unable to find data node for ", var , " used by
-      #   procedure node ", cmd@text, sep="")
-
-      # .ddg.insert.error.message(error.msg)
+      scope <- .ddg.get.scope(var, for.caller)
+      if (.ddg.data.node.exists(var, scope)) {
+        .ddg.data2proc(var, scope, cmd@abbrev)
+      }
     }
+    
+    # If the variable is inside an environment, like env$var, we also
+    # create nodes and edges for using the env.
+    var.env <- .ddg.extract.env (var, env)
+    if (!is.null(var.env)){
+      if (.ddg.data.node.exists(var.env, scope)) {
+        #print(".ddg.create.data.use.edges found data node for the environment")
+        .ddg.data2proc(var.env, scope, cmd@abbrev)
+      }
+      else {
+        scope <- .ddg.get.scope(var.env, for.caller)
+        if (.ddg.data.node.exists(var.env, scope)) {
+          .ddg.data2proc(var.env, scope, cmd@abbrev)
+        }
+      }
+    }
+    
   }
   #print (".ddg.create.data.use.edges Done")
 }
 
+#' .ddg.extract.var is given an expression that is either a variable
+#' or an expression with $ in it, like env$var or df$col and finds the part
+#' of it that represents a variable, returning the variable as
+#' a string.  If the $ operator is being used with an environment, like
+#' env$var, "env$var" is returned.  If it is the column of a data frame,
+#' like df$col, df is returned.
+#' @param var a string holding either a variable or an expression including
+#'   the $ operator
+#' @param env the environment in which var could be evaluated.  If NULL,
+#'   the environment is searched for.
+#' @return the portion of var that actually represents the variable
+#' @noRd 
+
+.ddg.extract.var <- function (var, env=NULL) {
+  # $ is used both to access variables in an environment and 
+  # columns in a data frame.  In the former case, we want the 
+  # qualified name env$var to be the thing being used.  For a 
+  # dataframe, we want to note that the data frame is being used.
+  #print (paste ("In .ddg.extract.var, var =", var))
+  if (stringi::stri_detect_fixed(var, "$")) {
+    #print ("Found $ in var")
+    parsed <- parse (text=var)[[1]]
+    if (parsed[[1]] == "$") {
+      #print ("First operator is $")
+      if (is.null(env)) {
+        env <- .ddg.get.env(deparse(parsed[[2]]))
+      }
+      
+      if (!is.environment(eval(parsed[[2]], env))) {
+        #print ("Not an environment")
+        var = deparse(parsed[[2]])
+      }
+    }
+  }
+  return(var)
+
+}
+
+#' .ddg.extract.env is given an expression that is either a variable
+#' or an expression with $ in it, like env$var or df$col and finds the part
+#' of it that represents the environment, returning it as
+#' a string.  If the $ operator is being used with an environment, like
+#' env$var, "env" is returned.  Otherwise NULL is returned.
+#' @param var a string holding either a variable or an expression including
+#'   the $ operator
+#' @param env the environment in which var could be evaluated.  If NULL,
+#'   the environment is searched for.
+#' @return the portion of var that actually represents the environment, or 
+#'   NULL if no environment is involved
+#' @noRd 
+
+.ddg.extract.env <- function (var, env=NULL) {
+  if (stringi::stri_detect_fixed(var, "$")) {
+    parsed <- parse (text=var)[[1]]
+    if (parsed[[1]] == "$") {
+      if (is.null(env)) {
+        env <- .ddg.get.env(deparse(parsed[[2]]))
+      }
+      
+      if (is.environment(eval(parsed[[2]], env))) {
+        return(deparse(parsed[[2]]))
+      }
+    }
+  }
+  return(NULL)
+}
+
+
+#.ddg.changing.env <- function (cmd, env) {
+#  print ("In .ddg.changing.env")
+#  vars.assigned <- cmd@vars.set
+#  env.to.set <- character()
+#  
+#  for (var in vars.assigned) {
+#    
+#    if (stringi::stri_detect_fixed(var, "$")) {
+#      print (paste ("Found $ in var", var))
+#      parsed <- parse (text=var)[[1]]
+#      if (parsed[[1]] == "$") {
+#        print ("First operator is $")
+#        arg2String <- deparse(parsed[[2]])
+#        if (is.null(env)) {
+#          env <- .ddg.get.env(arg2String)
+#        }
+#        
+#        arg2 <- eval(parsed[[2]], env)
+#        
+#        if (is.environment(arg2)) {
+#          print ("Found environment")
+#          if (!exists (deparse(parsed[[3]]), envir = arg2)) {
+#            print (paste (deparse(parsed[[3]], "is a new var in the environment")))
+#             env.to.set <- c(env.to.set, arg2String)  
+#          } 
+#        }
+#      }
+#    }
+#  }
+#  
+#  print (".ddg.changing.env is returning")
+#  print (env.to.set)
+#  return (env.to.set)
+#  
+#}
 
 #' .ddg.create.data.set.edges creates data nodes for 
 #' variables being set, saves the data, and creates edges from the
@@ -429,8 +548,8 @@
 #' @return nothing
 #' @noRd
 
-.ddg.create.data.set.edges <- function(vars.set, cmd, env) {
-  # print(paste("In .ddg.create.data.set.edges.for.cmd: cmd = ", cmd@abbrev))
+.ddg.create.data.set.edges <- function(vars.set, cmd, env, captured.output = NULL) {
+  #print(paste("In .ddg.create.data.set.edges.for.cmd: cmd = ", cmd@abbrev))
   vars.assigned <- cmd@vars.set
   
   for (var in vars.assigned) {
@@ -450,11 +569,43 @@
     }
     
     if (var != "") {
+      # Create the nodes and edges for setting the variable
+      var <- .ddg.extract.var (var, env)
       scope <- .ddg.get.scope(var, env=env)
       .ddg.save.var(var, env, scope)
       .ddg.proc2data(cmd@abbrev, var, scope)
+      
+      # If the variable is inside an environment, like env$var
+      # and var was not previously in the environment, we also
+      # create nodes and edges for change env.
+      var.env <- .ddg.extract.env (var, env)
+      if (!is.null(var.env)){
+        envList <- .ddg.get ("ddg.envList")
+        oldEnvContents <- envList[[var.env]]
+        
+        # Checks if this is the first variable set
+        if (is.null(oldEnvContents)) {
+          .ddg.save.var(var.env, env, scope)
+          .ddg.proc2data(cmd@abbrev, var.env, scope)
+        }
+        else {
+          # Checks if the variable being set was already in the environment
+          internal.var <- substring (var, stringi::stri_locate_first_fixed(var, "$")[1,"start"]+1)
+          if (!(internal.var %in% oldEnvContents)) {
+            .ddg.save.var(var.env, env, scope)
+            .ddg.proc2data(cmd@abbrev, var.env, scope)
+          }
+        }
+      }
     }
   }
+  
+  if (!is.null(captured.output) && length(captured.output) > 0) {
+    #cat(captured.output)
+    #print(paste("length(captured.output)",length(captured.output)))
+    .ddg.data.node("StandardOutput", "output", captured.output, "Standard output");
+    .ddg.proc2data(cmd@abbrev, "output", "Standard output")
+  }
 }
 
 #' .ddg.save.var
@@ -470,10 +621,19 @@
 .ddg.save.var <- function(var, env=NULL, scope=NULL) {
   if (is.null(env)) {
     env <- .ddg.get.env(var)
+    
+    # If no environment is found defining this variable, do not 
+    # save it.  It means that the variable is from a scope that
+    # is not available at the current line of code, such as a 
+    # non-local, yet not global scope.
+    if (is.null(env)) {
+      return()
+    }
   }
   if (is.null(scope)) {
     scope <- .ddg.get.scope(var, env=env)
   }
+
   
   # Special operators are defined by enclosing the name in `.  However,
   # the R parser drops those characters when we deparse, so when we parse
@@ -802,8 +962,14 @@
 
 .ddg.parse.commands <- function (exprs, script.name="", script.num=NA, environ, 
     ignore.patterns=c('^ddg.'), run.commands = FALSE, echo=FALSE, 
-    print.eval=echo, max.deparse.length=150, called.from.ddg.eval=FALSE, cmds=NULL) {
+    print.eval = echo, 
+    max.deparse.length=150, called.from.ddg.eval=FALSE, cmds=NULL, 
+    continue.echo=getOption("continue"), skip.echo = 0, prompt.echo=getOption("prompt"), 
+    spaced=FALSE, verbose=getOption("verbose"),
+    deparseCtrl = "showAttributes") {
 
+  
+  #print (paste ("In .ddg.parse.commands, exprs =", exprs))
   return.value <- NULL
   
   # Gather all the information that we need about the statements
@@ -861,6 +1027,10 @@
 
     # Loop over the commands as well as their string representations.
     for (i in 1:length(cmds)) {
+      # if-statement taken from R's source function
+      if (verbose) 
+        cat("\n>>>> eval(expression_nr.", i, ")\n\t\t =================\n")
+
       cmd <- cmds[[i]]
 
       if (.ddg.debug.lib()) print(paste(".ddg.parse.commands: Processing", cmd@abbrev))
@@ -905,17 +1075,30 @@
 
       # If the command does not match one of the ignored patterns.
       if (!any(sapply(ignore.patterns, function(pattern){grepl(pattern, cmd@text)}))) {
+        # Set to NULL so it is bound even if we are in console mode.
+        captured.output <- NULL
 
         # If sourcing, we want to execute the command.
         if (run.commands) {
-          # Print command.
+          # Print command.  The echo code is adapted from R's source function.
           if (echo) {
-            cmd.show <- 
-                paste0(substr(cmd@text, 
-                              1L, 
-                              min (max.deparse.length, nchar(cmd@text))), 
-                       "\n")
-            cat(cmd.show)
+            # lastshown holds the last line number shown to the user.
+            # When executing the first command, we can skip the number
+            # of lines at the beginning of the file, as specified
+            # by skip.echo.  This allows skipping over header comments.
+            if (i == 1) 
+              lastshown <- min(skip.echo, cmd@pos@startLine - 1)
+            if (lastshown < cmd@pos@endLine) {
+              # Look up the lines to display in the source file
+              srcrefs <- attr(exprs, "srcref")
+              srcref <- srcrefs[[i]]
+              srcfile <- attr(srcref, "srcfile")
+              dep <- getSrcLines(srcfile, lastshown + 1, cmd@pos@endLine)
+              lastshown <- cmd@pos@endLine
+            
+             .ddg.echo (dep, max.deparse.length, continue.echo, prompt.echo, spaced,
+                     cmd@pos@endLine - cmd@pos@startLine + 1)
+           }
           }
 
           # If we will create a node, then before execution, set
@@ -943,7 +1126,7 @@
           if (.ddg.debug.lib()) {
             print (paste (".ddg.parse.commands: Evaluating ", cmd@annotated))
           }
-
+          
           result <- withCallingHandlers(
           
               {
@@ -954,20 +1137,46 @@
                   # are executing an if-statement
                   if (grepl("^ddg|^.ddg|^prov", annot) 
                     || .ddg.get.statement.type(annot) == "if") {
-                      eval(annot, environ, NULL)
+                      captured.output <- .ddg.capture.output (returnWithVisible <- withVisible (eval(annot, environ, NULL)))
                       .ddg.set ("ddg.error.node.created", FALSE)
                   }
                   else {
-                    return.value <- eval(annot, environ, NULL)
+                    captured.output <- .ddg.capture.output (returnWithVisible <- withVisible (eval(annot, environ, NULL)))
                     #if (typeof(return.value) != "closure") {
                       #print (paste (".ddg.parse.commands: Done evaluating ", annot))
                       #print(paste(".ddg.parse.commands: setting ddg.last.R.value to", 
                       #            return.value))
                     #}
-                    .ddg.set ("ddg.last.R.value", return.value)
+                    .ddg.set ("ddg.last.R.value", returnWithVisible$value)
                     .ddg.set ("ddg.error.node.created", FALSE)
                   }
+                  
+                  #### Start code taken from R's source function. ####
+                  i.symbol <- mode(annot) == "name"
+                  if (!i.symbol) {
+                    curr.fun <- annot[[1L]]
+                    if (verbose) {
+                      cat("curr.fun:")
+                      utils::str(curr.fun)
+                    }
+                  }
+                  if (verbose >= 2) {
+                    cat(".... mode(ei[[1L]])=", mode(annot), "; paste(curr.fun)=")
+                    utils::str(paste(curr.fun))
+                  }
+                  # Print evaluation.
+                  if (print.eval && returnWithVisible$visible) {
+                    if (isS4(returnWithVisible$value))
+                      methods::show(returnWithVisible$value)
+                    else print(returnWithVisible$value)
+                  }
+                  if (verbose) 
+                    cat(" .. after ", sQuote(deparse(cmd@annotated, control = unique(c(deparseCtrl, 
+                                        "useSource")))), "\n", sep = "")
+                  #### End code taken from R's source function ####
                 }
+                
+                returnWithVisible
               },
             warning = .ddg.set.warning,
             error = function(e)
@@ -1034,10 +1243,7 @@
             cur.cmd.closed <- (ddg.cur.cmd.stack[stack.length] == "MATCHES_CALL")
             .ddg.pop.cmd ()
           }
-
-          # Print evaluation.
-          if (print.eval) print(result)
-        }
+        }  
 
         # Figure out if we should create a procedure node for this
         # command. We don't create it if it matches a last command
@@ -1089,7 +1295,7 @@
             }
           }
 
-          .ddg.create.data.use.edges(cmd, for.caller=FALSE)
+          .ddg.create.data.use.edges(cmd, for.caller=FALSE, d.environ)
 
           .ddg.create.file.read.nodes.and.edges()
           .ddg.link.function.returns(cmd)
@@ -1098,8 +1304,7 @@
             print(paste(".ddg.parse.commands: Adding input data nodes for", 
                         cmd@abbrev))
           }
-
-          .ddg.create.data.set.edges(vars.set, cmd, d.environ)
+          .ddg.create.data.set.edges(vars.set, cmd, d.environ, captured.output)
 
           if (.ddg.debug.lib()) {
             print(paste(".ddg.parse.commands: Adding output data nodes for", 
@@ -1131,6 +1336,17 @@
         }
       }
      }
+     
+     if (echo) {
+       # Output any extra lines from the file that are after the last line executed
+	     # This code is adapted from R's source function.  In the source function
+	     # they use the tail variable to identify statements done
+	     # after the last line of code is executed.
+       srcref <- attr(exprs, "wholeSrcref")
+       srcfile <- attr(srcref, "srcfile")
+        dep <- getSrcLines(srcfile, lastshown + 1, srcref[3])
+        .ddg.echo (dep, max.deparse.length, continue.echo, prompt.echo, spaced, 0)
+     }
 
      # Create a data node for each variable that might have been set in
      # something other than a simple assignment, with an edge from the
@@ -1153,6 +1369,109 @@
   #}
 }
 
+#' .ddg.capture.output is a simple re-write of R's capture.output
+#'  that allows us to capture the exception that occurs if the 
+#' connection is closed by the code whose output is being captured
+#' as is the case if that code includes a call to closeAllConnections.
+#' See capture.output for more details.
+#' @noRd 
+.ddg.capture.output <- function (...) 
+{
+  args <- substitute(list(...))[-1L]
+  type <- c("output", "message")
+  rval <- NULL
+  file <- textConnection("rval", "w", local = TRUE)
+  sink(file, type = type, split = TRUE)
+  on.exit({
+        sink(type = type, split = TRUE)
+        close(file)
+      })
+  pf <- parent.frame()
+  evalVis <- function(expr) withVisible(eval(expr, pf))
+  for (i in seq_along(args)) {
+    expr <- args[[i]]
+    tmp <- switch(mode(expr), expression = lapply(expr, evalVis), 
+        call = , name = list(evalVis(expr)), stop("bad argument"))
+    for (item in tmp) if (item$visible) 
+        print(item$value)
+  }
+  on.exit()
+  tryCatch(
+      {
+        sink(type = type, split = TRUE)
+        close(file)
+      },
+      warning = function (e) {}
+  )
+  if (is.null(rval)) 
+    invisible(NULL)
+  else rval
+}
+
+#' .ddg.echo prints the command to the screen
+#' 
+#' @param cmdText the lines to display.  This can include blank lines and comments that
+#'    preceded the command itself.
+#' @param max.deparse.length the maximum length to display
+#' @param continue.echo the prompt to put at the start of the condinuation lines of a multi-line statement
+#' @param prompt.echo the prompt to put at the start of the first line of a statement
+#' @param spaced if true, blank lines are inserted between statements executed
+#' @param curCmdLength the number of lines in the statement being executed.  This is the
+#'     length of the command itself, not including leading blank lines or comments.
+#' @noRd 
+.ddg.echo <- function (cmdText, max.deparse.length, continue.echo, prompt.echo, spaced, curCmdLength) {
+  # This function is extracted and adapted from R's source function.
+  dep <- cmdText
+  
+  # Remove leading blank lines
+  while (length(dep) && grepl("^[[:blank:]]*$", dep[1])) {
+    dep <- dep[-1]
+  }
+  
+  # No actual command.  This happens if we have reached the end of the file
+  # and there is a trailing comment.
+  if (curCmdLength == 0) {
+    # Print the trailing comments, with each line beginning with 
+    # the prompt character.
+    dep <- paste0(rep.int(prompt.echo, length(dep)), dep, 
+      collapse = "\n")
+  }
+  
+  else {
+    # Print the comments and first line of the command beginning with the
+    # prompt character.  If there is more than one line to the command,
+    # begin each of those with the continue character.
+    dep <- paste0(rep.int(c(prompt.echo, continue.echo), 
+            c(length(dep) - curCmdLength + 1, curCmdLength - 1)), dep, 
+            collapse = "\n")
+  }
+  
+  nd <- nchar(dep, "c")
+  
+  if (nd) {
+    # Truncate the output if it is too long
+    do.trunc <- nd > max.deparse.length
+    dep <- substr(dep, 1L, if (do.trunc) 
+              max.deparse.length
+            else nd)
+    sd <- "\""
+    nos <- "[^\"]*"
+    oddsd <- paste0("^", nos, sd, "(", nos, sd, nos, sd, 
+        ")*", nos, "$")
+    
+    # Insert blank line if spaced is true
+    cat(if (spaced) "\n", 
+        dep, 
+        if (do.trunc) 
+          # Make sure any open quotes are closed if the string
+          # is truncated.
+          paste(if (grepl(sd, dep) && grepl(oddsd, dep)) 
+                    " ...\" ..."
+                else " ....", 
+                "[TRUNCATED] "), "\n", sep = "")
+  }
+} 
+
 #' .ddg.evaluate.commands evaluates a list of parsed R statements. Provenance is 
 #' collected for inputs and outputs only. If an error or warning is generated 
 #' when a statement is evaluated, an output exception node is created containing 
@@ -1363,9 +1682,12 @@
     else {
       call.func <- as.character(call)
       # Ignore calls to ddg functions or to the functions that get called from 
-      # the outermost tryCatch to ddg code.
+      # the outermost tryCatch to ddg code.  Seems like a hack.  If provenance
+      # tools call prov.run, we need to add them to the list (like debug.init).
+      # We also could capture functions that actually are defined in a script if
+      # they start with "prov"!  
       if (!any (startsWith (call.func, c (".ddg", "ddg", "prov", "doTryCatch", 
-        "tryCatch")))) {
+        "tryCatch", "debug.init")))) {
           if (for.caller && !script.func.found) {
             script.func.found <- TRUE
           }
@@ -1423,7 +1745,9 @@
   stopifnot(!is.null(fnum))
 
   tryCatch (
-    if(!exists(name, sys.frame(fnum), inherits=TRUE)) return(NULL),
+    if(!exists(name, sys.frame(fnum), inherits=TRUE)) {
+      return(NULL)
+    },
     error = function(e) {}
   )
   env <- .ddg.where(name, sys.frame(fnum))
